#include <stdio.h>
#include <stdlib.h>
#include <time.h>

typedef struct 
{
   int w,h;
   unsigned char** data;
}Image;

Image* make_image(int w, int h)
{
   Image* I = malloc(sizeof(Image));
   I->w = w;
   I->h = h;
   I->data = malloc(h*sizeof(unsigned char*));
   int i = 0;
   for ( ; i < h; ++i)
      I->data[i] = malloc(w*sizeof(unsigned char));
   return I;
}

void clean_image(Image* I)
{
   int i = 0;
   for ( ; i < I->h; ++i)
      free(I->data[i]);
   free(I->data);
}

void delete_image(Image* I)
{
   clean_image(I);
   free(I);
}

Image* xadrez(int w, int h, int size, unsigned char cor1, unsigned char cor2)
{
   Image* I = make_image(size*w, size*h);
   unsigned char cor = cor1;
   int i, j;
   int is_even = w % 2 == 0 ;
   for (i = 0; i < h; ++i)
   {
      for (j = 0; j < w; ++j)
      {
         int k, l;
         for (k = 0 ; k < size; ++k)
            for (l = 0 ; l < size; ++l)
               I->data[size*i+k][size*j+l] = cor;
         cor = (cor == cor1) ? cor2 : cor1;
      }
      if ( is_even ) cor = (cor == cor1) ? cor2 : cor1;
   }
   return I;
}

Image* random_image(int w, int h)
{
   Image* I = make_image(w, h);
   time_t seconds;
   time(&seconds);
   srand((unsigned int) seconds);
   int i, j;
   for (i = 0; i < h; ++i)
   {
      for (j = 0; j < w; ++j)
      {
         I->data[i][j] = rand() % 256;
      }
   }
   return I;
}

void write_image(Image* I, const char* filename)
{
   FILE* file = fopen(filename, "w");
   fprintf(file,"P5\n");
   fprintf(file,"%d %d\n255\n", I->w, I->h);
   fclose(file);
   file = fopen(filename, "ab");
   int i, j;
   for (i = 0 ; i < I->h; ++i)
   {
      for (j = 0 ; j < I->w; ++j)
         fprintf(file,"%c", I->data[i][j]);
      /*fprintf(file,"\n");*/
   }   
   fclose(file);
}

Image* read_image(const char* filename)
{
   FILE* file = fopen(filename, "r");
   char buffer[50];
   fscanf(file,"%s", buffer);
   if (buffer[1] != '5')
   {
      fprintf(stderr, "Tipo de arquivo errado!\n");
      return NULL;
   }
   int w, h;
   fscanf(file,"%d %d", &w, &h);
   fclose(file);
   Image* I = make_image(w,h);
   file = fopen(filename, "rb");
   size_t i, j;
   char* line = NULL;
   for (i = 0; i < 3; ++i)
      getline(&line, &j, file);
   free(line);
   for (i = 0 ; i < I->h; ++i)
   {
      for (j = 0 ; j < I->w; ++j)
         fscanf(file,"%c", &(I->data[i][j]));
   }   
   fclose(file);
   return I;
}

int main(int argc, char** argv)
{
   /*Image* R = random_image(512,512);*/
   /*write_image(R, "random.pgm");*/
   /*delete_image(R);*/
   /*Image* X = xadrez(8,8,70,255,0);*/
   /*write_image(X, "xadrez.pgm");*/
   /*delete_image(X);*/

   if (argc != 2) 
   {
      fprintf(stderr, "Numero de argumentos errado\n");
      fprintf(stderr, "Uso: %s nome_do_arquivo\n", argv[0]);
      exit(EXIT_FAILURE);
   }
   Image* R = read_image(argv[1]);
   if (R != NULL)
   {
      write_image(R, "xadrez_new.pgm");
      delete_image(R);
      exit(EXIT_SUCCESS);
   }
   else
      exit(EXIT_FAILURE);
}
